#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <unistd.h>
#include "state.h"
#include "io.h"
#include "mooremachine.h"
#include "coffee_machine.h"


static char stdin_buffer[256];

void initialize(moore_t* m, void *p)
{
  int i;
  
  //Intializing inputs
  init_input(m, IN_START_BUTTON_PIN, IN_START_BUTTON);
  init_input(m, IN_CUP_READY_PIN, IN_CUP_READY);

  //Initializing outputs
  init_output(m, OUT_CUP_PIN, OUT_CUP);
  init_output(m, OUT_COFFEE_PIN, OUT_COFFEE);
  init_output(m, OUT_READY_LED_PIN, OUT_READY_LED);
  
  //Registering states
  state_t* standby = create_state(STANDBY, standby_next_state, standby_output);
  state_t* supplycup = create_state(SUPPLY_CUP, supplycup_next_state, supplycup_output);
  state_t* supplycoffee = create_state(SUPPLY_COFFEE, supplycoffee_next_state, supplycoffee_output);
  state_t* ready = create_state(READY, ready_next_state, ready_output);
  
  register_state(m, STANDBY, standby);
  register_state(m, SUPPLY_CUP, supplycup);
  register_state(m, SUPPLY_COFFEE, supplycoffee);
  register_state(m, READY, ready);
  
  set_start_state(m,STANDBY);
}

void init_input(moore_t* m, pin_t pin, input_id_t id)
{
	input_t* in = (input_t*)malloc(sizeof(input_t));
  in->value = 0;
  in->pin = pin;
  in->id = id;
  register_input(m,id,in,read_input);
}

void init_output(moore_t* m, pin_t pin, output_id_t id)
{
	output_t* out = (output_t*)malloc(sizeof(output_t));
  out->value = 0;
  out->pin = pin;
  out->id = id;
  register_output(m,id,out,write_output);
}

/*STATE FUNCTIONS*/
void* standby_next_state(input_t** inputs, void** states, void* par)
{
	bool val_button;
	bool val_cup;
	val_button = inputs[IN_START_BUTTON]->value;
	val_cup = inputs[IN_CUP_READY]->value;
	if(!val_cup && val_button)
		return states[SUPPLY_CUP];
	return states[STANDBY];
}

void* supplycup_next_state(input_t** inputs, void** states, void* par)
{
	bool val_cup;
	val_cup = inputs[IN_CUP_READY]->value;
	if(val_cup)
		return states[SUPPLY_COFFEE];
	return states[SUPPLY_CUP];
}

void* supplycoffee_next_state(input_t** inputs, void** states, void* par)
{
	usleep(TIMEOUT);
	return states[READY];
}

void* ready_next_state(input_t** inputs, void** states, void* par)
{
	bool val_cup;
	val_cup = inputs[IN_CUP_READY]->value;
	if(!val_cup)
		return states[STANDBY];
	return states[READY];
}

void standby_output(output_t** outputs, void* par)
{
  system("clear");
  printf("State: Standby\n");
	outputs[OUT_CUP]->value = 0;
	outputs[OUT_COFFEE]->value = 0;
	outputs[OUT_READY_LED]->value = 0;
}

void supplycup_output(output_t** outputs, void* par)
{
  system("clear");
  printf("State: Suppling Cup\n");
	outputs[OUT_CUP]->value = 1;
	outputs[OUT_COFFEE]->value = 0;
	outputs[OUT_READY_LED]->value = 0;
}

void supplycoffee_output(output_t** outputs, void* par)
{
  system("clear");
  printf("State: Suppling Coffee\n");
	outputs[OUT_CUP]->value = 0;
	outputs[OUT_COFFEE]->value = 1;
	outputs[OUT_READY_LED]->value = 0;
}

void ready_output(output_t** outputs, void* par)
{
  system("clear");
  printf("\nState: Ready\n");
	outputs[OUT_CUP]->value = 0;
	outputs[OUT_COFFEE]->value = 0;
	outputs[OUT_READY_LED]->value = 1;
}


void read_input(input_t* in)
{
  in->value = 0;
  if((in->id == IN_START_BUTTON && strchr(stdin_buffer,'a'))
  	|| (in->id == IN_CUP_READY && strchr(stdin_buffer,'s')))
  	 in->value = 1;
	printf("Input %d - %d\n", in->id, in->value);
}

void write_output(output_t* out)
{
		printf("Output %d - %d\n", out->id, out->value);
}

void begin_cycle(void* p)
{
	printf("Send inputs >>> "); 
	gets(stdin_buffer);
}