#ifndef COFFEE_MACHINE_H
#define COFFEE_MACHINE_H

#include <stdlib.h>
#include <stdbool.h>
#include "state.h"
#include "io.h"
#include "mooremachine.h"

#define TIMEOUT 1000000
#define WAIT_TIME 1500000

#define NUM_OF_STATES 4
#define NUM_OF_INPUTS 2
#define NUM_OF_OUTPUTS 3

/* I/O Pin Definition  */
#define IN_START_BUTTON_PIN 21
#define IN_CUP_READY_PIN 22
#define OUT_CUP_PIN 17
#define OUT_COFFEE_PIN 19
#define OUT_READY_LED_PIN 20

/* States */
#define STANDBY        0
#define SUPPLY_CUP     1
#define SUPPLY_COFFEE  2
#define READY          3

/* Outputs */
#define OUT_CUP       0
#define OUT_COFFEE	  1
#define OUT_READY_LED 2

/* inputs */
#define IN_START_BUTTON 0
#define IN_CUP_READY    1

void begin_cycle(void* p);

/*
* Function to initialize io
*/
void init_output(moore_t* m, pin_t pin, output_id_t id);
void init_input(moore_t* m, pin_t pin, input_id_t id);

/*
* Functions to read and write io
*/
void read_input(input_t* in);
void write_output(output_t* out);

//Init function
void initialize(moore_t* m, void *p);

/*
* State transition functions related to the states of the coffee machine
*/
void* standby_next_state(input_t** inputs, void** states, void* par);
void* supplycup_next_state(input_t** inputs, void** states, void* par);
void* supplycoffee_next_state(input_t** inputs, void** states, void* par);
void* ready_next_state(input_t** inputs, void** states, void* par);

/*
* Output functions relates to the states of the coffee machine
*/
void standby_output(output_t** outputs, void* par);
void supplycup_output(output_t** outputs, void* par);
void supplycoffee_output(output_t** outputs, void* par);
void ready_output(output_t** outputs, void* par);

#endif /* COFFEE_MACHINE_H */
