#include <stdlib.h>
#include <stdio.h>
#include <stdbool.h>
#include <assert.h>
#include <unistd.h>
#include <stdint.h>
#include <sys/time.h>
#include <math.h>
#include "state.h"
#include "io.h"
#include "mooremachine.h"

//Private functions
uint64_t get_time();
static void update_state(moore_t* m);
static void read_inputs(moore_t* m);
static void write_outputs(moore_t* m);

#ifdef RASPI
#define ST_BASE (0x20003000)
#define TIMER_OFFSET (4)
static uint64_t* timer = NULL;
static bool init_timer();
#endif

//Main Cycle
void run(moore_t* m, cycle_fun begin_run, void* p)
{
	uint64_t time = 0;
  uint64_t time_diff = 0;
  int64_t time_wait = 0;
  while(true)
  {
  	time = get_time();
    begin_run(p);
    read_inputs(m);
    update_state(m);
    write_outputs(m);
    
    //Ensures that the duration of each cycle is cycle_limit
    time_diff = get_time() - time;
    //Busy wait
    while(m->cycle_limit - time_diff > 0)
    	time_diff = get_time() - time;
  }
}

void update_state(moore_t* m)
{
	m->current_state = (state_t*)m->current_state->next_state(m->inputs, (void*)m->states, NULL);
	printf("New State %d\n", m->current_state->id);
}

void write_outputs(moore_t* m)
{
	int i;
	m->current_state->update_output(m->outputs,NULL);
	for(i = 0; i < m->num_outputs; i++)
		m->outputs[i].write(&(m->outputs[i]));
}

void read_inputs(moore_t* m)
{
  int i;
  for(i = 0; i < m->num_inputs; i++)
		m->inputs[i].read(&(m->inputs[i]));
}

//Funzione per la lettura del tempo
#ifdef RASPI
static bool init_timer()
{
	int timer_fd = -1;
	void *stimer_base = NULL;

	  // get access to system core memory
    if (-1 == (timer_fd = open("/dev/mem", O_RDONLY))) {
        fprintf(stderr, "open() failed.\n");
        return false;
    }
    
    // map a specific page into process's address space
    if (MAP_FAILED == (stimer_base = mmap(NULL, 4096,
                        PROT_READ, MAP_SHARED, timer_fd, ST_BASE))) {
        fprintf(stderr, "mmap() failed.\n");
        return false;
    }
  	
  	// set up pointer, based on mapped page
    timer = (long long int *)((char *)st_base + TIMER_OFFSET);
    
    return true;
}

static uint64_t get_time()
{
	return *timer;
}
#else
static uint64_t get_time() {
    struct timeval tv;
    gettimeofday(&tv,NULL);
    return tv.tv_sec*(uint64_t)1000000+tv.tv_usec;
}
#endif
